﻿local Engine, Api, Locale, Settings = unpack(Speechless)

-- Constants
local MIN_PERFORMANCE_MILLISECONDS = 0
local MAX_PERFORMANCE_MILLISECONDS = 1000
local STEP_PERFORMANCE_MILLISECONDS = 50

local Configuration = Engine:GetModule("Configuration")
local IsReloadRequired = false
local Handlers = {}

-- Register the module's configuration options panel.
function Configuration:OnRegisterOptions()
	local panel = self.Component
	
	-- The name of the AddOn or group of configuration options. This is
	-- the text that will display in the AddOn options list.
	panel.name = Locale.CONFIG_OPTIONS_NAME

	-- Name of the parent of the AddOn or group of configuration
	-- options. This identifies "panel" as the child of another category.
	-- If the parent category doesn't exist, "panel" will be displayed as
	-- a regular category.
	panel.parent = nil

	panel.okay = Configuration.OnOkay
	panel.cancel = Configuration.OnCancel
	panel.default = Configuration.OnDefault
	panel.refresh = Configuration.OnRefresh

	-- Description
	local title = Configuration:CreateTitle({ Panel = panel, Text = Locale.ADDON_NAME })
	local subText = Configuration:CreateSubText({ Panel = panel, Anchor = title, Text = Locale.ADDON_NAME })

	-- DisableDialog = true,
	-- DisableDialogUnlessNew = true,
	-- DisableAudio = true,
	-- DisableAudioUnlessNew = true,
	
	local api = Settings:Get("Api")

	-- Controls: Disable Dialog
	local disableDialog = Configuration:CreateCheckBox({ Panel = panel, Name = string.format("%s%sDisableDialogCheckButton", Engine.AddOnName, self.Name), Checked = api.DisableDialog, Text = Locale.CONFIG_OPTIONS_DISABLE_DIALOG_TEXT, Tooltip = Locale.CONFIG_OPTIONS_DISABLE_DIALOG_TOOLTIP })
	disableDialog:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", -2, -8)
	disableDialog:SetScript("OnClick", function(self)
		Configuration:ReloadRequired()
	end)
	disableDialog.OnOkay = function(self)
		api.DisableDialog = self:GetChecked()
	end
	disableDialog.OnRefresh = function(self)
		self:SetValue(api.DisableDialog)
	end
	Configuration:AddHandler(disableDialog)

	-- Controls: Disable Dialog Unless New
	local disableDialogUnlessNew = Configuration:CreateCheckBox({ Panel = panel, Name = string.format("%s%sDisableDialogUnlessNewCheckButton", Engine.AddOnName, self.Name), Checked = api.DisableDialogUnlessNew, Text = Locale.CONFIG_OPTIONS_DISABLE_DIALOG_UNLESS_NEW_TEXT, Tooltip = Locale.CONFIG_OPTIONS_DISABLE_DIALOG_UNLESS_NEW_TOOLTIP })
	disableDialogUnlessNew:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", 20, -30)
	disableDialogUnlessNew:SetScript("OnClick", function(self)
		Configuration:ReloadRequired()
	end)
	disableDialogUnlessNew.OnOkay = function(self)
		api.DisableDialogUnlessNew = self:GetChecked()
	end
	disableDialogUnlessNew.OnRefresh = function(self)
		self:SetValue(api.DisableDialogUnlessNew)
	end
	Configuration:AddHandler(disableDialogUnlessNew)
	Configuration:AddDependency(disableDialog, disableDialogUnlessNew)

	-- Controls: Disable Audio
	local disableAudio = Configuration:CreateCheckBox({ Panel = panel, Name = string.format("%s%sDisableAudioCheckButton", Engine.AddOnName, self.Name), Checked = api.DisableAudio, Text = Locale.CONFIG_OPTIONS_DISABLE_AUDIO_TEXT, Tooltip = Locale.CONFIG_OPTIONS_DISABLE_AUDIO_TOOLTIP })
	disableAudio:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", -2, -52)
	disableAudio:SetScript("OnClick", function(self)
		Configuration:ReloadRequired()
	end)
	disableAudio.OnOkay = function(self)
		api.DisableAudio = self:GetChecked()
	end
	disableAudio.OnRefresh = function(self)
		self:SetValue(api.DisableAudio)
	end
	Configuration:AddHandler(disableAudio)

	-- Controls: Disable Audio Unless New
	local disableAudioUnlessNew = Configuration:CreateCheckBox({ Panel = panel, Name = string.format("%s%sDisableAudioUnlessNewCheckButton", Engine.AddOnName, self.Name), Checked = api.DisableAudioUnlessNew, Text = Locale.CONFIG_OPTIONS_DISABLE_AUDIO_UNLESS_NEW_TEXT, Tooltip = Locale.CONFIG_OPTIONS_DISABLE_AUDIO_UNLESS_NEW_TOOLTIP })
	disableAudioUnlessNew:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", 20, -74)
	disableAudioUnlessNew:SetScript("OnClick", function(self)
		Configuration:ReloadRequired()
	end)
	disableAudioUnlessNew.OnOkay = function(self)
		api.DisableAudioUnlessNew = self:GetChecked()
	end
	disableAudioUnlessNew.OnRefresh = function(self)
		self:SetValue(api.DisableAudioUnlessNew)
	end
	Configuration:AddHandler(disableAudioUnlessNew)
	Configuration:AddDependency(disableAudio, disableAudioUnlessNew)

	-- Controls: Instances Only
	local instancesOnly = Configuration:CreateCheckBox({ Panel = panel, Name = string.format("%s%sInstancesOnlyCheckButton", Engine.AddOnName, self.Name), Checked = api.InstancesOnly, Text = Locale.CONFIG_OPTIONS_INSTANCES_ONLY_TEXT, Tooltip = Locale.CONFIG_OPTIONS_INSTANCES_ONLY_TOOLTIP })
	instancesOnly:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", -2, -110)
	instancesOnly:SetScript("OnClick", function(self)
		Configuration:ReloadRequired()
	end)
	instancesOnly.OnOkay = function(self)
		api.InstancesOnly = self:GetChecked()
	end
	instancesOnly.OnRefresh = function(self)
		self:SetValue(api.InstancesOnly)
	end
	Configuration:AddHandler(instancesOnly)

	InterfaceOptions_AddCategory(panel)
end

-- This method will run when the player clicks "okay" in the Interface
-- Options.
function Configuration:OnOkay()
	-- Apply changes to elements
	for i = 1, #Handlers do
		Handlers[i]:OnOkay()
	end

	if IsReloadRequired then
		-- Confirm with player to reload the UI
		StaticPopup_Show("IMMERSIVE_RELOADUI_CONFIRM")
	end
end

-- This method will run when the player clicks "cancel" in the Interface
-- Options. Use this to revert their changes.
function Configuration:OnCancel()
end

-- This method will run when the player clicks "defaults". Use this to
-- revert their changes to your defaults.
function Configuration:OnDefault()
end

-- This method will run when the Interface Options frame calls its OnShow
-- function and after defaults have been applied via the panel.default
-- method described above. Use this to refresh your panel's UI in case
-- settings were changed without player interaction.
function Configuration:OnRefresh()
	for i = 1, #Handlers do
		Handlers[i]:OnRefresh()
	end
end

-- Indicates that a change has been made to the settings that will required
-- the UI to be reloaded.
function Configuration:ReloadRequired()
	IsReloadRequired = true
end

-- Adds an element option category to the cache.
function Configuration:AddHandler(handler)
	table.insert(Handlers, handler)
end

-- Creates a dependency between a control and an enable/disable checkbox.
function Configuration:AddDependency(parent, child)
	if parent.Children == nil then
		parent.Children = {}
		parent:SetScript("OnClick", function(self)
			for i = 1, #self.Children do
				if self:GetChecked() then
					self.Children[i]:Enable()
				else
					self.Children[i]:Disable()
				end
			end
		end)
	end

	table.insert(parent.Children, child)
	if parent:GetChecked() then
		child:Enable()
	else
		child:Disable()
	end
end

-- Prompt asking the player to reload the UI.
StaticPopupDialogs["IMMERSIVE_RELOADUI_CONFIRM"] = {
	text = Locale.CONFIG_OPTIONS_RELOAD_REQUIRED,
	button1 = Locale.CONFIG_OPTIONS_RELOAD_REQUIRED_YES,
	button2 = Locale.CONFIG_OPTIONS_RELOAD_REQUIRED_NO,
	OnAccept = function()
		ReloadUI()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3
}